// build qtr curve.jsx
//
// Build a photoshop curve for a QTR digital negative correction curve by automatically reading step wedge patches
//
// DJE	11/12/2008	Prototype	
// DJE  03/28/2009	Make sure curve X values, which are actually wedge measurements monotonically increase.  If a pair doesn't, toss it.

// enable double clicking from the Macintosh Finder or the Windows Explorer
#target photoshop

// Execute startup stub.
main() ;

// Startup stub.
function main()
{
	try
	{
    	// Basic error handing.  
    	if (app.documents.length == 0)
    	{
    		alert("No step wedge image currently loaded") ;
    		return 0 ;
    	}
    	if (app.activeDocument.mode != DocumentMode.GRAYSCALE)
    	{
    		alert("Step wedge must be a Gray Scale image") ;
    		return 0;
    	}
	    // Run the body of the script
        doit() ;
    }
    catch (e)
    {
         alert('Unexpected error: (' + e + ') on line ' + e.line);	
		 return 0 ;
    }
}
    
// Does the work.
function doit() 
{
    // Stepwedge height and width.  Note this will be in the "users" units; pixels, inch, cm, etc.
    var h = app.activeDocument.height
    var w = app.activeDocument.width ;
     
    // Approximate eyeballed centers of the gray patches in pixels, convert these to percent so we can deal with units in inch, cm, pixels or whatever the user has setup in photoshop.
    // This assumes the step wedge has been cropped and straightened.
    var wcenters = Array(70,193,290,370,440,520,600,670,740,820,900,980,1060,1140,1220,1300,1360,1440,1530,1630,1740) ;
    var wsize = 1800 ;
    var i ;
    for (i = 0; i < wcenters.length; i++)
    {
        wcenters[i] = wcenters[i] / wsize ;	
    }
    var vcenter  = 225 / 461 ;
    
    // Work on the active document
    var ref = app.activeDocument ;
     
    // Duplicate the background so we keep the orginal intact
    var blur = ref.activeLayer.duplicate()  ;
    
    // Apply a guassian blur
    blur.name = "blur=levels"
    blur.applyGaussianBlur(6) ;
    
    // Get a color sampler for the current image
    var sampler ;
    if (ref.colorSamplers.length == 0)
    {
    	sampler = ref.colorSamplers.add(Array(wcenters[0] * w, vcenter * h)) ;
    }
    else
    {
        sampler = ref.colorSamplers[0] ;	
    	sampler.move(Array(wcenters[0] * w, vcenter * h)) ;
    }

    // Sample the 100% and 0% patches and apply a levels correction
    var black = ((100 - sampler.color.gray.gray) / 100) * 255;
    sampler.move(Array(wcenters[wcenters.length-1] * w, vcenter * h)) ;
    var white = ((100 - sampler.color.gray.gray) / 100) * 255 ;
    blur.adjustLevels(black, white, 1.0, 0, 255) ;
    
    // Measure the 100% though 0% patches of the step wedge in 10% increments.
    var measurements = Array() ;
    var j ;
    for (i = 0, j = 0; i <= 20; i += 2, j++)
    {
         sampler.move(Array(wcenters[i] * w, vcenter * h)) ;
         measurements[j] = sampler.color.gray.gray ;
    }

    // Build list of input/output pairs for the curves layer.  
    var pairtext = ""
    var pairs = new Array() ;
	var j = 0 ;
    for (i = 0; i < measurements.length; i++)
    {
        var pair = new Array() ;
        pair[0] = measurements[i] / 100 * 255 ;
        pair[1] = (100 - (i * 10)) / 100 * 255 ;
  		
	if (j == 0 || (j && pair[0] < pairs[j-1][0]))
            pairs[j++] = pair ;
    }
    
    // Now the bizzare magic.  You can't create a curves layer without using an "action".   Actions are butt ugly in the javascript API and almost impossible to understand.
    // What I did was use the script listerner to record creating a curves layer.  I then put the resultant cyber-crud in the function below and gingerly edited it to
    // feed in my curve values.  
    ref.activeLayer = blur ;
    curvesLayer(pairs) ; 

    // Pop Dialog to show text version curve
    pairtext = "" ;
    for (i = 0 ; i < pairs.length; i++)
    {
         pairtext = pairtext + String(100-(pairs[i][0] * 100 / 255.0)) + ";" + String(100-(pairs[i][1] * 100 / 255.0)) + " " ;
    }
    pairtext = String("\"") + pairtext + String("\"") ;
    var dlg = new Window('dialog', "Gray Curve");  
    dlg.panel = dlg.add('panel', undefined, "Curve");  
    dlg.panel.alignChildren = "fill";  
    dlg.panel_text1 = dlg.panel.add('edittext', [0, 0,600, 100], pairtext, {multiline:true});  
    var closeBtn = dlg.add('button',undefined,"Close");  
  
    closeBtn.onClick = function() {  
       dlg.close();  
    };  
    
    dlg.show() ;
    
    return 1 ;
}

// Bizzare action to create a curves layer using javascript.  Pretty neat eh...
function curvesLayer(xypairs)
{
var id233 = charIDToTypeID( "Mk  " );
    var desc52 = new ActionDescriptor();
    var id234 = charIDToTypeID( "null" );
        var ref43 = new ActionReference();
        var id235 = charIDToTypeID( "AdjL" );
        ref43.putClass( id235 );
    desc52.putReference( id234, ref43 );
    var id236 = charIDToTypeID( "Usng" );
        var desc53 = new ActionDescriptor();
        var id237 = charIDToTypeID( "Type" );
            var desc54 = new ActionDescriptor();
            var id238 = stringIDToTypeID( "presetKind" );
            var id239 = stringIDToTypeID( "presetKindType" );
            var id240 = stringIDToTypeID( "presetKindCustom" );
            desc54.putEnumerated( id238, id239, id240 );
            var id241 = charIDToTypeID( "Adjs" );
                var list3 = new ActionList();
                    var desc55 = new ActionDescriptor();
                    var id242 = charIDToTypeID( "Chnl" );
                        var ref44 = new ActionReference();
                        var id243 = charIDToTypeID( "Chnl" );
                        var id244 = charIDToTypeID( "Chnl" );
                        var id245 = charIDToTypeID( "Blck" );
                        ref44.putEnumerated( id243, id244, id245 );
                    desc55.putReference( id242, ref44 );
                    var id246 = charIDToTypeID( "Crv " );
                        var list4 = new ActionList();
                        // Ike started here
					    var i ;
                        for (i = xypairs.length-1; i >= 0; i--)
                        {
                            var desc56 = new ActionDescriptor();
                            var id247 = charIDToTypeID( "Hrzn" );
                            desc56.putDouble( id247, xypairs[i][0] );

                            var id248 = charIDToTypeID( "Vrtc" );
                            desc56.putDouble( id248, xypairs[i][1] );

                            var id249 = charIDToTypeID( "Pnt " );
                            list4.putObject( id249, desc56 );
                       }
                       // Ike ended here
                       desc55.putList( id246, list4 ) ;
                var id256 = charIDToTypeID( "CrvA" );
                list3.putObject( id256, desc55 );
            desc54.putList( id241, list3 );
        var id257 = charIDToTypeID( "Crvs" );
        desc53.putObject( id237, id257, desc54 );
    var id258 = charIDToTypeID( "AdjL" );
    desc52.putObject( id236, id258, desc53 );
executeAction( id233, desc52, DialogModes.NO);
}
